import { AxiosInstance } from 'axios'
import {
  EditKlinkDetails,
  KlinkDetails,
  KlinkList,
  ViewKlinkDetails,
} from '../../common/types'
import AuthManager from './auth-manager'
import log from 'electron-log'
import { createClient } from './client'

export default class KlinkManager {
  private client: AxiosInstance

  constructor(private authManager: AuthManager) {
    this.authManager = authManager
    this.client = createClient(authManager)
  }

  async create(klinkRequest: KlinkDetails): Promise<string> {
    log.info('[Klink] Creating klink')

    try {
      const result = await this.client.post('/v1/tokens', {
        title: klinkRequest.title,
        data: klinkRequest.body,
        sharedWith: klinkRequest.sharedWith,
        notifications: klinkRequest.notifications,
      })

      return result.data.token
    } catch (e) {
      if (e.response && e.response.status === 401) {
        throw new Error('No access to create Klinks, try logging in again')
      }

      if (e.response && e.response.status === 400) {
        throw new Error('Data not accepted by server')
      }

      log.error(`[Klink] Unable to create Klink: ${e.message}`)

      throw new Error('Unable to access Klink storage')
    }
  }

  async modify(token: string, klinkRequest: KlinkDetails): Promise<void> {
    log.info('[Klink] Modifying klink')

    try {
      await this.client.put(`/v1/tokens/${token}`, {
        title: klinkRequest.title,
        data: klinkRequest.body,
        sharedWith: klinkRequest.sharedWith,
        notifications: klinkRequest.notifications,
      })
    } catch (e) {
      if (e.response && e.response.status === 401) {
        throw new Error('No access to modify this klink, try logging in again')
      }

      if (e.response && e.response.status === 400) {
        throw new Error('Data not accepted by server')
      }

      log.error(`[Klink] Unable to modify Klink: ${e.message}`)

      throw new Error('Unable to access Klink storage')
    }
  }

  async read(token: string): Promise<ViewKlinkDetails | EditKlinkDetails> {
    log.info('[Klink] Reading klink')

    try {
      const { data: klinkData } = await this.client.get(`/v1/tokens/${token}`)
      const { id, url, data, title, sharedWith, notifications } = klinkData

      if (klinkData.editable) {
        return { editable: true, token, url, title, data, sharedWith, notifications }
      }

      return { editable: false, token, title, data, url }
    } catch (e) {
      if (e.response && e.response.status === 404) {
        throw new Error('Klink not found')
      }

      log.error(`[Klink] Unable to read Klink: ${e.message}`)

      throw new Error('Unable to access Klink storage')
    }
  }

  async getCurrentUserKlinks(page?: number): Promise<KlinkList> {
    log.info('[Klink] Reading current user klinks')

    try {
      const response = await this.client.get(`/v1/tokens?page=${page}`)

      return response.data
    } catch (e) {
      log.error(`[Klink] Unable to read Klinks: ${e.message}`)

      throw new Error('Unable to access Klink storage')
    }
  }
}
