import axios from 'axios'
import jwtDecode, { JwtPayload } from 'jwt-decode'
import * as url from 'url'
import * as qs from 'qs'
import * as keytar from 'keytar'
import {
  AUTH0_DOMAIN,
  REDIRECT_URI,
  CLIENT_ID,
  HEADER_AUTHORIZATION,
} from '../config.json'
import { CurrentUserDetails, PhoneNumber } from '../../common/types'

type decodedJwtToken = JwtPayload & PhoneNumber
type FetchAuthDataBody = {
  grant_type: string
  code?: string
  redirect_uri?: string
  refresh_token?: string
}
type AuthResponse = {
  access_token: string
  expires_in: number
  refresh_token: string
}

const REFRESH_TRESHOLD = 30000

const APP_SERVICE = 'kloke-app'

export let currentUser: CurrentUserDetails | null = null

const callbacks: (() => void)[] = []

export const onOauthChange = (callback: () => void): void => {
  callbacks.push(callback)
}

function executeCallbacks() {
  callbacks.forEach((callback) => callback())
}

export function getCurrentUser() {
  return currentUser
}

export function shouldRefreshTokens() {
  if (currentUser === null) {
    return false
  } else {
    const { expirationDate } = currentUser
    const expirationDateWithTreshold = new Date(
      expirationDate.getTime() - REFRESH_TRESHOLD
    )
    return expirationDateWithTreshold && expirationDateWithTreshold < new Date()
  }
}

export function getAuthenticationURL() {
  const queryParams = {
    client_id: CLIENT_ID,
    redirect_uri: REDIRECT_URI,
    scope: 'local_tokens',
    response_type: 'code',
  }

  return `${AUTH0_DOMAIN}/oauth?${qs.stringify(queryParams)}`
}

async function fetchAuthData(body: FetchAuthDataBody) {
  const options = {
    headers: {
      Authorization: `Basic ${HEADER_AUTHORIZATION}`,
    },
  }

  try {
    const response = await axios.post(
      `${AUTH0_DOMAIN}/oauth/v2/token`,
      qs.stringify(body),
      options
    )

    const { data } = response
    setUserData(data)

    executeCallbacks()
  } catch (error) {
    await logout()
    throw error
  }
}

function setUserData(data: AuthResponse) {
  const { access_token, expires_in, refresh_token } = data
  const { phone_number } = jwtDecode<decodedJwtToken>(access_token)

  currentUser = {
    accessToken: access_token,
    expirationDate: new Date(new Date().getTime() + expires_in * 1000),
    phoneNumber: phone_number,
  }

  if (refresh_token) {
    keytar.setPassword(APP_SERVICE, 'account', refresh_token)
  }
}

export async function refreshTokens() {
  const refreshToken = await keytar.getPassword(APP_SERVICE, 'account')
  if (!currentUser) {
    throw new Error('No available refresh token.')
  }

  if (refreshToken) {
    const body = {
      grant_type: 'refresh_token',
      refresh_token: refreshToken,
    }

    fetchAuthData(body)
  }
}

export async function loadTokens(callbackURL: string) {
  const urlParts = url.parse(callbackURL, true)
  const query = urlParts.query

  const body = {
    grant_type: 'authorization_code',
    code: `${query.code}`,
    redirect_uri: REDIRECT_URI,
  }

  fetchAuthData(body)
}

export async function logout() {
  await keytar.deletePassword(APP_SERVICE, 'accoount')
  currentUser = null
  executeCallbacks()
}
